#include <WiFi.h>
#include <ESPAsyncWebServer.h>
#include <ESPmDNS.h>

// --- Pin Definitions ---
#define PHASE_A_PIN 4
#define ENABLE_A_PIN 5
#define PHASE_B_PIN 6
#define ENABLE_B_PIN 7

// --- WiFi Configuration ---
static const char* WIFI_SSID = "ATTsmith";
static const char* WIFI_PASS = "rolltide9172";

// --- mDNS Configuration ---
static const char* MDNS_HOSTNAME = "maestrostepper";
static const char* CAMERA_MDNS_HOSTNAME = "maestrocamera";

// --- Global Variables ---
AsyncWebServer server(80);
volatile int currentStep = 0;
int step_delay = 10;
bool isMotorRunning = false;

// Define logic levels
#define COIL_ENABLE HIGH
#define COIL_DISABLE LOW

void setup() {
  Serial.begin(115200);

  // Connect to WiFi
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  Serial.print("Connecting to hamburger...");
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.print(".");
  }
  Serial.println("\nConnected to WiFi!");
  Serial.print("IP Address: ");
  Serial.println(WiFi.localIP());

  // Setup motor pins
  pinMode(PHASE_A_PIN, OUTPUT);
  pinMode(ENABLE_A_PIN, OUTPUT);
  pinMode(PHASE_B_PIN, OUTPUT);
  pinMode(ENABLE_B_PIN, OUTPUT);

  isMotorRunning = false;

  // mDNS
  if (!MDNS.begin(MDNS_HOSTNAME)) {
    Serial.println("Error setting up MDNS responder!");
    while (1) delay(1000);
  }
  Serial.println("mDNS responder started: http://" + String(MDNS_HOSTNAME) + ".local");
  MDNS.addService("http", "tcp", 80);

  // HTML page
  server.on("/", HTTP_GET, [](AsyncWebServerRequest* request) {
    String html = R"rawliteral(
<!DOCTYPE html>
<html>
<head>
  <title>Stepper and Camera Control</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <style>
    body { font-family: Arial, sans-serif; padding: 15px; }
    h1, h2 { text-align: center; color: #333; }
    .control-section, .camera-section {
      margin-bottom: 20px;
      padding: 15px;
      border: 1px solid #ddd;
      border-radius: 5px;
      background-color: #f9f9f9;
    }
    .button-container { text-align: center; margin-top: 15px; }
    button {
      padding: 10px 20px;
      font-size: 16px;
      margin: 5px;
      cursor: pointer;
      border: none;
      border-radius: 4px;
      color: white;
    }
    .start-button { background-color: #4CAF50; }
    .stop-button { background-color: #f44336; }
    .start-button:hover { background-color: #45a049; }
    .stop-button:hover { background-color: #da190b; }
    #status { text-align: center; font-weight: bold; margin-top: 10px; font-size: 1.1em; }
    img { display: block; margin: 10px auto; max-width: 100%; height: auto; border: 1px solid #ccc; }
  </style>
</head>
<body>
  <h1>Stepper & Camera Control</h1>

  <div class="control-section">
    <h2>Stepper Control</h2>
    <div id="status">Motor Status: Stopped</div>
    <div class="button-container">
      <button class="start-button" onclick="sendCmd('/start')">Start Motor</button>
      <button class="stop-button" onclick="sendCmd('/stop')">Stop Motor</button>
    </div>
  </div>

  <div class="camera-section">
    <h2>Camera View</h2>
    <img id="cameraStream" src="http://maestrocamera.local/continuous.mjpeg" width="640" height="480" alt="Camera Stream Loading...">
  </div>

  <script>
    function sendCmd(url) {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', url, true);
      xhr.onload = function() {
        if (xhr.status === 200) {
          console.log('Command sent: ' + url);
          var statusElement = document.getElementById('status');
          if (url === '/start') {
            statusElement.textContent = 'Motor Status: Running';
            statusElement.style.color = 'green';
          } else if (url === '/stop') {
            statusElement.textContent = 'Motor Status: Stopped';
            statusElement.style.color = 'red';
          }
        } else {
          console.error('Error sending command: ' + xhr.statusText);
          alert('Failed to send command.');
        }
      };
      xhr.onerror = function() {
        console.error('Request failed');
        alert('Request failed. Could not reach the server.');
      };
      xhr.send();
    }

    document.addEventListener('DOMContentLoaded', () => {
      var statusElement = document.getElementById('status');
      statusElement.textContent = 'Motor Status: Stopped';
      statusElement.style.color = 'red';

      var img = document.getElementById('cameraStream');
      img.onerror = function() {
        this.alt = 'Camera stream failed to load. Check camera connection and hostname.';
        this.style.border = '2px dashed red';
      };
    });
  </script>
</body>
</html>
)rawliteral";
    request->send(200, "text/html", html);
  });

  // Start and stop motor routes
  server.on("/start", HTTP_GET, [](AsyncWebServerRequest* request) {
    Serial.println("Received command: /start");
    isMotorRunning = true;
    request->send(200, "text/plain", "OK, Motor Started");
  });


  server.on("/start", HTTP_GET, [](AsyncWebServerRequest* request) {
    Serial.println("Received command: /start");
    isMotorRunning = true;
    request->send(200, "text/plain", "OK, Motor Started");
  });


  server.begin();
  Serial.println("HTTP server started");
}

void setMotorStep(int step) {
  int phase = step % 4;
  if (phase < 0) phase += 4;

  switch (phase) {
    case 0:
      digitalWrite(PHASE_A_PIN, HIGH);
      digitalWrite(ENABLE_A_PIN, HIGH);
      digitalWrite(PHASE_B_PIN, HIGH);
      digitalWrite(ENABLE_B_PIN, HIGH);
      break;
    case 1:
      digitalWrite(PHASE_A_PIN, LOW);
      digitalWrite(ENABLE_A_PIN, HIGH);
      digitalWrite(PHASE_B_PIN, HIGH);
      digitalWrite(ENABLE_B_PIN, HIGH);
      break;
    case 2:
      digitalWrite(PHASE_A_PIN, LOW);
      digitalWrite(ENABLE_A_PIN, HIGH);
      digitalWrite(PHASE_B_PIN, LOW);
      digitalWrite(ENABLE_B_PIN, HIGH);
      break;
    case 3:
      digitalWrite(PHASE_A_PIN, HIGH);
      digitalWrite(ENABLE_A_PIN, HIGH);
      digitalWrite(PHASE_B_PIN, LOW);
      digitalWrite(ENABLE_B_PIN, HIGH);
      break;
  }
}


void loop() {
  if (isMotorRunning) {
    currentStep++;
    setMotorStep(currentStep);
    delay(step_delay);
  } else {
    delay(10);
  }
}
